-----------------------------------------------------------------------------------------------
-- Client Lua Script for GuildMail
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- GuildMail Module Definition
-----------------------------------------------------------------------------------------------
local GuildMail = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function GuildMail:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.strMessage = ""
	self.strRank = ""
	self.strSubject = ""
	self.nLocation = 0
	self.bRank = false
	self.bSending = true
	self.tList = {}
	--list entry
	--{ strName = name, strRank = rank }

    return o
end

function GuildMail:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"Mail"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- GuildMail OnLoad
-----------------------------------------------------------------------------------------------
function GuildMail:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("GuildMail.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- GuildMail OnDocLoaded
-----------------------------------------------------------------------------------------------
function GuildMail:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "GuildMailForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
	
		self.wndHelp = self.wndMain:FindChild("HelpAndInfo")
		self.wndHelp:Show(false, true)


		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("guildmail", "OnGuildMailOn", self)
		Apollo.RegisterSlashCommand("mailguild", "OnGuildMailOn", self)
		Apollo.RegisterEventHandler("GuildRoster", "OnGuildRoster", self)	--for getting list of guild members
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)	--for getting an interface button
		Apollo.RegisterEventHandler("ToggleGuildMail", "OnToggleGuildMail", self)


		-- Do additional Addon initialization here
		for i,guild in ipairs(GuildLib.GetGuilds()) do	--actual guild member request
      		guild:RequestMembers()
   		end
		self:InitializeHooks()		--hook into other addons
	end
end

-----------------------------------------------------------------------------------------------
-- GuildMail Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/guildmail" and "/mailguild"
function GuildMail:OnGuildMailOn()
	self.wndMain:FindChild("SubjectInput"):SetText(self.strSubject)	--if there's already data, put it in
	self.wndMain:FindChild("MessageInput"):SetText(self.strMessage)
	self.wndMain:FindChild("RankNameInput"):SetText(self.strRank)
	self.wndMain:FindChild("RankCheck"):SetCheck(self.bRank)
	self.wndMain:Invoke() -- show the window
end

function GuildMail:OnInterfaceMenuListHasLoaded()		--for interface menu button
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "GuildMail", {"ToggleGuildMail", "", "IconSprites:Icon_Windows32_UI_CRB_InterfaceMenu_Mail"})
	--self:UpdateInterfaceMenuAlerts()
end

function GuildMail:OnToggleGuildMail()
	if self.wndMain:IsShown() then
		self:OnCancel()
	else
		self:OnGuildMailOn()
	end
end

function GuildMail:OnGuildRoster(guild, roster)
	if guild:GetType() == GuildLib.GuildType_Guild then	--we don't care about circles
		self.tList = {}		--empty the list
		local tRanks = guild:GetRanks()	--getting rank names
		local count = 0
		for i,member in ipairs(roster) do	--look through the guild members
			local item = {}
			item.strName = member.strName
			item.strRank = tRanks[member.nRank].strName
			if (not self.bRank or item.strRank == self.strRank) and item.strName ~= GameLib.GetPlayerUnit():GetName() then	--we don't want the player on the list
				count = count + 1
				self.tList[count] = item	--store the member's info
			end
		end
	end
end

function GuildMail:InitializeHooks()
	local addonMail = Apollo.GetAddon("Mail")
	local addonGuildRoster = Apollo.GetAddon("GuildContentRoster")	--we want to interact with these two addons
	
	local fnOldComposeMail = addonMail.ComposeMail	--save the function to call later
	addonMail.ComposeMail = function(tMail)
		fnOldComposeMail(tMail)	--call the original function, don't want to completely undo it
		local btnNext = Apollo.LoadForm(self.xmlDoc, "NextMember", tMail.luaComposeMail.wndMain, self)	--insert the button
		btnNext:Show(false, true)
		
		local fnOldOnEmailSent = tMail.luaComposeMail.OnEmailSent	--the mail composition screen is a sub-addon to mail
		tMail.luaComposeMail.OnEmailSent = function(tMailCompose, wndHandler, wndControl, bSuccess)
			fnOldOnEmailSent(tMailCompose, wndHandler, wndControl, bSuccess)
			if self.bSending then
				self:FillMailInfo()	--if we are still sending then continue filling info
			end
		end
	end
	
	if addonGuildRoster ~= nil then	--we want to make sure it exists. addons like viragsocial override guildcontentroster
		local fnOldInitialize = addonGuildRoster.Initialize
		addonGuildRoster.Initialize = function(tGuildRoster, wndParent)
			fnOldInitialize(tGuildRoster, wndParent)	--call the old function and add the button, like before
			local btnOpen = Apollo.LoadForm(self.xmlDoc, "GuildMailOpen", tGuildRoster.tWndRefs.wndMain:FindChild("AdvancedOptionsContainer"), self)
			btnOpen:Show(true)
		end
	end
end

function GuildMail:FillMailInfo()
	local addonMail = Apollo.GetAddon("Mail")
	addonMail:ComposeMail()	--open mail window and create the sub-addon
	local addonCompose = addonMail.luaComposeMail
	local wndCompose = addonCompose.wndMain
	wndCompose:FindChild("NextMember"):Show(true, true)	--show the button we added
	local strRecipient = self:GetRecipient()
	if strRecipient ~= nil then	--if there's another recipient, fill in info
		wndCompose:FindChild("NameEntryText"):SetText(strRecipient)
		wndCompose:FindChild("SubjectEntryText"):SetText(self.strSubject)
		wndCompose:FindChild("MessageEntryText"):SetText(self.strMessage)
	else	--if there's no more recipients, close the window as normal
		wndCompose:Close()
		Print("GuildMail ending")
	end
end

function GuildMail:GetRecipient()
	self.nLocation = self.nLocation + 1
	if self.nLocation > table.getn(self.tList) then	--if we are past the end, we aren't sending anymore
		self.bSending = false
		return nil
	end
	local recipient = self.tList[self.nLocation]	--get the next recipient
	if recipient ~= nil then
		return recipient.strName
	else
		return nil
	end
end


-----------------------------------------------------------------------------------------------
-- GuildMailForm Functions
-----------------------------------------------------------------------------------------------
function GuildMail:OnBeginSending()
	for i,guild in ipairs(GuildLib.GetGuilds()) do	--get guild member list
      	guild:RequestMembers()
   	end
	self.bSending = true	--we are sending now
	self.nLocation = 0	--start at the beginning of the list
	Print("GuildMail: Loaded "..table.getn(self.tList).." guild members")
	self:FillMailInfo()	--begin filling info
end

-- when the Cancel button is clicked
function GuildMail:OnCancel()
	self.wndMain:Close() -- hide the window
end

function GuildMail:OnMailMessageChanged( wndHandler, wndControl, strText )	--the next few functions are from the xml
	self.strMessage = strText
end

function GuildMail:OnSubjectChanged( wndHandler, wndControl, strText )
	self.strSubject = strText
end

function GuildMail:OnRankChecked( wndHandler, wndControl, eMouseButton )
	self.bRank = true
end

function GuildMail:OnRankUnchecked( wndHandler, wndControl, eMouseButton )
	self.bRank = false
end

function GuildMail:OnRankNameChanged( wndHandler, wndControl, strText )
	self.strRank = strText
end

function GuildMail:OnNextMemberPressed()
	Apollo.GetAddon("Mail").luaComposeMail.wndMain:Close()	--we have to close and reopen the window to make it work
	self:FillMailInfo()
end

function GuildMail:OnHelpAndInfo()
	self.wndHelp:Show(not self.wndHelp:IsShown())
end

function GuildMail:OnCloseHelpAndInfo()
	self.wndHelp:Show(false, true)
end

-----------------------------------------------------------------------------------------------
-- GuildMail Instance
-----------------------------------------------------------------------------------------------
local GuildMailInst = GuildMail:new()
GuildMailInst:Init()
